package T4Chart;

import java.awt.Color;
import java.text.SimpleDateFormat;
import java.util.Vector;
import java.sql.Timestamp;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.StandardLegend;
import org.jfree.chart.axis.DateAxis;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.StandardXYItemRenderer;
import org.jfree.chart.renderer.RangeType;
import org.jfree.chart.renderer.xy.XYItemRenderer;
import org.jfree.data.xy.XYDataset;
import org.jfree.data.general.SeriesException;
import org.jfree.data.time.Second;
import org.jfree.data.time.Minute;
import org.jfree.data.time.TimeSeries;
import org.jfree.data.time.TimeSeriesCollection;
import org.jfree.ui.ApplicationFrame;
import org.jfree.ui.RefineryUtilities;
import org.jfree.ui.Spacer;
import org.jfree.util.Log;
import org.jfree.util.PrintStreamLogTarget;

import java.text.NumberFormat;
import java.text.ParseException;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.labels.StandardXYToolTipGenerator;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.StackedXYAreaRenderer;
import org.jfree.chart.renderer.xy.XYAreaRenderer;
import org.jfree.data.xy.DefaultTableXYDataset;
import org.jfree.data.xy.XYSeries;
import org.jfree.chart.renderer.xy.XYDifferenceRenderer;
import org.jfree.chart.axis.DateAxis;
import org.jfree.chart.axis.ValueAxis;
/**
 *
 */
public class T4ChartChart extends javax.swing.JFrame {
    /**
     *
     * @param title  the frame title.
     */
    public T4ChartChart(final String[] lineName, final SqlIterator[] it,
                        boolean isStacked, boolean isDifference) {
        super("T4Chart");
        final XYDataset dataset;
        if (isDifference)
            dataset = createDifferenceDataset(lineName, it);
        else
            dataset = createDataset(lineName, it);
        final JFreeChart chart;
        if (isStacked)
            chart = createStackedChart("", dataset, isStacked);
        else if (isDifference)
            chart = createDifferenceChart("", dataset);
        else
            chart = createStackedChart("", dataset, isStacked);
        final ChartPanel chartPanel = new ChartPanel(chart);
        chartPanel.setPreferredSize(new java.awt.Dimension(750, 405));
        chartPanel.setMouseZoomable(true, false);
        setContentPane(chartPanel);

    }

    private JFreeChart OldcreateChart(final String title, final XYDataset dataset, boolean isStacked) {

        final JFreeChart 
            chart = ChartFactory.createTimeSeriesChart(
                        title, "Timestamp", "Value", dataset,
                        true, true, false);

        chart.setBackgroundPaint(Color.white);

        final StandardLegend sl = (StandardLegend)chart.getLegend();
        sl.setDisplaySeriesShapes(true);

        final XYPlot plot = chart.getXYPlot();
        plot.setBackgroundPaint(Color.lightGray);
        plot.setDomainGridlinePaint(Color.white);
        plot.setRangeGridlinePaint(Color.white);
        plot.setAxisOffset(new Spacer(Spacer.ABSOLUTE, 5.0, 5.0, 5.0, 5.0));
        plot.setDomainCrosshairVisible(true);
        plot.setRangeCrosshairVisible(true);
        
        final XYItemRenderer renderer = plot.getRenderer();
        if (renderer instanceof StandardXYItemRenderer) {
            final StandardXYItemRenderer rr = (StandardXYItemRenderer)renderer;
//            rr.setPlotShapes(true);
//            rr.setShapesFilled(true);
//            System.err.println(rr.getRangeType());
            rr.setItemLabelsVisible(true);
        }
        
        final DateAxis axis = (DateAxis) plot.getDomainAxis();
        axis.setDateFormatOverride(new SimpleDateFormat("dd HH:mm:ss"));
        
        return chart;

    }

    private JFreeChart createStackedChart(final String title, final XYDataset dataset, boolean isStacked) {
        final StandardXYToolTipGenerator toolTipGenerator = 
            new StandardXYToolTipGenerator(
                    StandardXYToolTipGenerator.DEFAULT_TOOL_TIP_FORMAT,
                    new SimpleDateFormat("dd HH:mm:ss"), NumberFormat.getInstance()
                );
        final DateAxis xAxis = new DateAxis(null);
        xAxis.setLowerMargin(0.0);
        xAxis.setUpperMargin(0.0);
        
        final NumberAxis yAxis = new NumberAxis(null);
        yAxis.setAutoRangeIncludesZero(true);

        final XYItemRenderer renderer;
        if (isStacked) {
            renderer = new StackedXYAreaRenderer(XYAreaRenderer.AREA, toolTipGenerator, null);        
            ((StackedXYAreaRenderer)renderer).setOutline(true);
        } else {
            renderer = new StandardXYItemRenderer(StandardXYItemRenderer.LINES, toolTipGenerator, null);
        }
        
        renderer.setItemLabelsVisible(true);

        final XYPlot plot = new XYPlot(dataset, xAxis, yAxis, renderer);
        plot.setBackgroundPaint(Color.lightGray);
        plot.setDomainGridlinePaint(Color.white);
        plot.setRangeGridlinePaint(Color.white);
        plot.setAxisOffset(new Spacer(Spacer.ABSOLUTE, 5.0, 5.0, 5.0, 5.0));
        plot.setDomainCrosshairVisible(true);
        plot.setRangeCrosshairVisible(true);
        
        final DateAxis axis = (DateAxis) plot.getDomainAxis();
        axis.setDateFormatOverride(new SimpleDateFormat("dd HH:mm:ss"));

        final JFreeChart chart = new JFreeChart(null, JFreeChart.DEFAULT_TITLE_FONT, plot, true);

        return chart;
    }

    private JFreeChart createDifferenceChart(String title, final XYDataset dataset) {
        final JFreeChart chart = ChartFactory.createTimeSeriesChart(
            null,
            null, null,
            dataset,
            true,  // legend
            true,  // tool tips
            false  // URLs
        );
//        chart.setBackgroundPaint(Color.white);
       
        final XYPlot plot = chart.getXYPlot();
        final XYDifferenceRenderer renderer = new XYDifferenceRenderer(Color.red, Color.blue, false);
//        final XYDifferenceRenderer renderer = new XYDifferenceRenderer();
//        renderer.setItemLabelsVisible(true);
        plot.setRenderer(renderer);
        plot.setBackgroundPaint(Color.lightGray);
        plot.setDomainGridlinePaint(Color.white);
        plot.setRangeGridlinePaint(Color.white);
        plot.setAxisOffset(new Spacer(Spacer.ABSOLUTE, 5.0, 5.0, 5.0, 5.0));
        plot.setDomainCrosshairVisible(true);
        plot.setRangeCrosshairVisible(true);
        
        final ValueAxis domainAxis = new DateAxis(null);
        domainAxis.setLowerMargin(0.0);
        domainAxis.setUpperMargin(0.0);
        plot.setDomainAxis(domainAxis);
        plot.setForegroundAlpha(0.5f);  
        return chart;      
    }


    /**
     * Creates a dataset
     *
     * @return the dataset.
     */
    private XYDataset createDataset(final String[] lineTitle, final SqlIterator iter[]) {
        final DefaultTableXYDataset dataset = new DefaultTableXYDataset();
        final SimpleDateFormat sdf = new SimpleDateFormat("yyyy-M-d H:m:s.S");

        for (int i = 0; i < iter.length; ++i) {
            SqlIterator it = iter[i];
            XYSeries s1 = new XYSeries(lineTitle[i], true, false);
            // warning we need exactly the same set of X value (Timestamp) for all series
            // if not graph is badly draw
            // we got record which have s.S part equal to 00.0 (see Database class)
            while(it.hasNext()) {
                Vector res = (Vector)it.next();
                Timestamp ts = (Timestamp)res.get(0);
                Double v = (Double)res.get(1);
//                System.err.print(ts);
//                System.err.print(" ");
//                System.err.println(v);
                try {
                    s1.add(sdf.parse(ts.toString()).getTime(), v);
                }
                catch (ParseException e) {
                    e.printStackTrace();
                }
                catch (SeriesException e) {
                    e.printStackTrace();
                }
            }
            it.close();
            dataset.addSeries(s1);
        }
        
        return dataset;
    }
    
    /**
     * Creates a dataset
     *
     * @return the dataset.
     */
    private XYDataset createDifferenceDataset(final String[] lineTitle, final SqlIterator iter[]) {
        final TimeSeriesCollection dataset = new TimeSeriesCollection();
        SqlIterator it0 = iter[0];
        SqlIterator it1 = iter[1];
        TimeSeries s0 = new TimeSeries(lineTitle[0], Minute.class);
        TimeSeries s1 = new TimeSeries(lineTitle[1], Minute.class);
        Minute minute = new Minute(0, 0, 1, 1, 2000);
        while(it0.hasNext() && it1.hasNext()) {
            Vector res0 = (Vector)it0.next();
            Vector res1 = (Vector)it1.next();
            Double v0 = (Double)res0.get(1);
            Double v1 = (Double)res1.get(1);
            s0.add(minute, v0);
            s1.add(minute, v1);
            minute = (Minute)minute.next();
        }
        it0.close();
        it1.close();

        dataset.addSeries(s0);
        dataset.addSeries(s1);
        
        dataset.setDomainIsPointsInTime(true);
        
        return dataset;
 }
}
